package main

import "core:fmt"
import "core:math"
import rl "vendor:raylib"

Player :: struct {
	using entity: Entity,
	cameraOffset: Vec3,
	direction:    f32,
	yaw:          f32,
	bob_timer:    f64,
	bob_position: Vec3,
	bob_factor:   f32,
	camera:       rl.Camera3D,
}

player_sword_tex: rl.Texture
player_sword_mesh: rl.Mesh
player_sword_mat: rl.Material

player_load_resources :: proc() {
	player_sword_tex = TEXTURES["sword.png"] // rl.LoadTexture("sword.png")
	player_sword_mesh = rl.GenMeshPlane(1.5, 3.6, 1, 1)
	player_sword_mat = rl.LoadMaterialDefault()
	player_sword_mat.shader = discard_shader
	rl.SetMaterialTexture(&player_sword_mat, rl.MaterialMapIndex.ALBEDO, player_sword_tex)
}

player_unload_resources :: proc() {
	rl.UnloadMaterial(player_sword_mat)
	rl.UnloadMesh(player_sword_mesh)
}

player_create :: proc() -> Player {
	return(
		Player {
			size = {0.5, 2, 0.5},
			cameraOffset = {0, 1.8, 0},
			camera = rl.Camera3D {
				up = {0, 1, 0},
				fovy = 80,
				target = {0, 0, 10},
				projection = rl.CameraProjection.PERSPECTIVE,
			},
		} \
	)
}

player_update :: proc(using player: ^Player, delta: f32) {
	using rl
	bob_timer += f64(delta)
	mouseDelta := GetMouseDelta()

	direction -= mouseDelta.x * 0.003
	yaw -= mouseDelta.y * 0.003

	yaw = Clamp(yaw, -math.PI / 2 + 0.001, math.PI / 2 - 0.001)

	dir_right := direction + math.PI / 2
	forward := Vec3{math.sin(direction), 0, math.cos(direction)}
	right := Vec3{math.sin(dir_right), 0, math.cos(dir_right)}

	motion := Vec3{}

	if IsKeyDown(KeyboardKey.W) {motion += forward * 0.1}
	if IsKeyDown(KeyboardKey.S) {motion -= forward * 0.1}
	if IsKeyDown(KeyboardKey.A) {motion += right * 0.1}
	if IsKeyDown(KeyboardKey.D) {motion -= right * 0.1}
	motion = rl.Vector3ClampValue(motion, 0, 0.2)
	if Vector3Length(motion) != 0 {
		bob_factor = min(bob_factor + delta * 3, 1)
	} else {
		bob_factor = max(bob_factor - delta * 5, 0)
	}
	velocity.xz = Vector3MoveTowards(velocity, motion * 100, 100 * delta).xz
	position += velocity * delta

	// camera positioning, targetting and bobbing
	bob_position = Vec3{0, math.sin_f32(f32(bob_timer * math.PI * 4)), 0} * bob_factor * 0.3
	bob_position = Vector3RotateByAxisAngle(bob_position, Vec3{0, 1, 0}, direction)
	camera.position = position + cameraOffset + bob_position
	target := Vec3 {
		math.sin(direction) * math.cos(yaw),
		math.sin(yaw),
		math.cos(direction) * math.cos(yaw),
	}
	camera.target = position + cameraOffset + target + bob_position
}

player_draw :: proc(using player: ^Player) {
	using rl

	rlPushMatrix()
	offset := position + cameraOffset + (bob_position / 2)
	rlTranslatef(offset.x, offset.y, offset.z)
	rlRotatef(math.to_degrees(player.direction), 0, 1, 0)
	rlRotatef(math.to_degrees(-player.yaw), 1, 0, 0)
	rlTranslatef(-1, -0.3, 1)
	rlTranslatef(math.sin_f32(f32(bob_timer * math.PI * 2 + math.PI / 4)) * bob_factor * 0.3, 0, 0)
	rlRotatef(-90, 1, 0, 0)
	rlRotatef(-90, 0, 0, 1)
	rlRotatef(210, 0, 1, 0)
	rlScalef(0.3, 0.3, 0.3)
	DrawMesh(player_sword_mesh, player_sword_mat, rl.Matrix(1))

	rlPopMatrix()
}
